@@ -1,11 +1,11 @@
 /* -*-pgsql-c-*- */
 /*
- * $Header: /cvsroot/pgpool/pgpool-II/main.c,v 1.20 2007/11/09 04:37:35 y-asaba Exp $
+ * $Header: /cvsroot/pgpool/pgpool-II/main.c,v 1.45.2.1 2009/03/08 14:58:15 gleu Exp $
  *
  * pgpool: a language independent connection pool server for PostgreSQL 
  * written by Tatsuo Ishii
  *
- * Copyright (c) 2003-2007	PgPool Global Development Group
+ * Copyright (c) 2003-2009	PgPool Global Development Group
  *
  * Permission to use, copy, modify, and distribute this software and
  * its documentation for any purpose and without fee is hereby
@@ -53,6 +53,9 @@
 #include "parser/pool_memory.h"
 #include "parser/pool_string.h"
 
+/*
+ * Process pending signal actions.
+ */
 #define CHECK_REQUEST \
 	do { \
 		if (wakeup_request) \
@@ -93,6 +96,7 @@ static void reload_config(void);
 static int pool_pause(struct timeval *timeout);
 static void pool_sleep(unsigned int second);
 static void kill_all_children(int sig);
+static int get_next_master_node(void);
 
 static RETSIGTYPE exit_handler(int sig);
 static RETSIGTYPE reap_handler(int sig);
@@ -102,6 +106,7 @@ static RETSIGTYPE health_check_timer_handler(int sig);
 static RETSIGTYPE wakeup_handler(int sig);
 
 static void usage(void);
+static void show_version(void);
 static void stop_me(void);
 
 static int trigger_failover_command(int node, const char *command_line);
@@ -109,7 +114,14 @@ static int trigger_failover_command(int node, const char *command_line);
 static struct sockaddr_un un_addr;		/* unix domain socket path */
 static struct sockaddr_un pcp_un_addr;  /* unix domain socket path for PCP */
 
-ProcessInfo *pids;	/* child pid table */
+ProcessInfo *pids;	/* shmem child pid table */
+
+/*
+ * shmem connection info table
+ * this is a two dimension array. i.e.:
+ * con_info[pool_config->num_init_children][pool_config->max_pool]
+ */
+ConnectionInfo *con_info;		
 
 static int unix_fd;	/* unix domain socket fd */
 static int inet_fd;	/* inet domain socket fd */
@@ -125,7 +137,7 @@ static int exiting = 0;		/* non 0 if I'm exiting */
 static int switching = 0;		/* non 0 if I'm fail overing or degenerating */
 
 #ifdef NOT_USED
-static int degenerated = 0;	/* set non 0 if already degerated */
+static int degenerated = 0;	/* set non 0 if already degenerated */
 #endif
 
 static int clear_cache = 0;		/* non 0 if clear chache option (-c) is given */
@@ -138,7 +150,7 @@ long int weight_master;	/* normalized weight of master (0-RAND_MAX range) */
 
 static int stop_sig = SIGTERM;	/* stopping signal default value */
 
-static int health_check_timer_expired;		/* non 0 if health check timer expired */
+static volatile sig_atomic_t health_check_timer_expired;		/* non 0 if health check timer expired */
 
 POOL_REQUEST_INFO *Req_info;		/* request info area in shared memory */
 volatile sig_atomic_t *InRecovery; /* non 0 if recovery is started */
@@ -163,7 +175,6 @@ int main(int argc, char **argv)
 	int i;
 	int pid;
 	int size;
-	ConnectionInfo *con_info = NULL;
 	int retrycnt;
 	int sys_retrycnt;
 
@@ -174,7 +185,7 @@ int main(int argc, char **argv)
 	snprintf(pcp_conf_file, sizeof(pcp_conf_file), "%s/%s", DEFAULT_CONFIGDIR, PCP_PASSWD_FILE_NAME);
 	snprintf(hba_file, sizeof(hba_file), "%s/%s", DEFAULT_CONFIGDIR, HBA_CONF_FILE_NAME);
 
-	while ((opt = getopt(argc, argv, "a:cdf:F:hm:n")) != -1)
+	while ((opt = getopt(argc, argv, "a:cdf:F:hm:nv")) != -1)
 	{
 		switch (opt)
 		{
@@ -241,6 +252,10 @@ int main(int argc, char **argv)
 				not_detach = 1;
 				break;
 
+			case 'v':
+				show_version();
+				exit(0);
+
 			default:
 				usage();
 				exit(1);
@@ -280,7 +295,7 @@ int main(int argc, char **argv)
 
 				if (kill(pid, SIGHUP) == -1)
 				{
-					pool_error("could not stop pid: %d. reason: %s", pid, strerror(errno));
+					pool_error("could not reload configuration file pid: %d. reason: %s", pid, strerror(errno));
 					pool_shmem_exit(1);
 					exit(1);
 				}
@@ -336,7 +351,7 @@ int main(int argc, char **argv)
 
 	if (pool_semaphore_create(MAX_NUM_SEMAPHORES))
 	{
-		pool_error("Unable to create semaphoes. Exiting...");
+		pool_error("Unable to create semaphores. Exiting...");
 		pool_shmem_exit(1);
 		exit(1);
 	}
@@ -402,7 +417,7 @@ int main(int argc, char **argv)
 	/* initialize Req_info */
 	Req_info->kind = NODE_UP_REQUEST;
 	memset(Req_info->node_id, -1, sizeof(int) * MAX_NUM_BACKENDS);
-	Req_info->master_node_id = 0;
+	Req_info->master_node_id = get_next_master_node();
 	Req_info->conn_counter = 0;
 
 	InRecovery = pool_shared_memory_create(sizeof(int));
@@ -467,19 +482,19 @@ int main(int argc, char **argv)
 
 			if (retrycnt == 0)
 			{
-				pool_log("starting health checking");
+				pool_debug("starting health checking");
 			}
 			else
 			{
-				pool_log("retrying %d th health checking", retrycnt);
+				pool_debug("retrying %d th health checking", retrycnt);
 			}
 
 			if (pool_config->health_check_timeout > 0)
 			{
 				/*
 				 * set health checker timeout. we want to detect
-				 * commnuication path failure much earlier before
-				 * TCP/IP statck detects it.
+				 * communication path failure much earlier before
+				 * TCP/IP stack detects it.
 				 */
 				pool_signal(SIGALRM, health_check_timer_handler);
 				alarm(pool_config->health_check_timeout);
@@ -490,7 +505,9 @@ int main(int argc, char **argv)
 			 */
 			errno = 0;
 			health_check_timer_expired = 0;
+			POOL_SETMASK(&UnBlockSig);
 			sts = health_check();
+			POOL_SETMASK(&BlockSig);
 			if (pool_config->parallel_mode || pool_config->enable_query_cache)
 				sys_sts = system_db_health_check();
 
@@ -567,9 +584,10 @@ int main(int argc, char **argv)
 			for (;;)
 			{
 				int r;
+				struct timeval t = {3, 0};
 
 				POOL_SETMASK(&UnBlockSig);
-				r = pool_pause(NULL);
+				r = pool_pause(&t);
 				POOL_SETMASK(&BlockSig);
 				if (r > 0)
 					break;
@@ -580,31 +598,49 @@ int main(int argc, char **argv)
 	pool_shmem_exit(0);
 }
 
+static void show_version(void)
+{
+	fprintf(stderr, "%s version %s (%s)\n",	PACKAGE, VERSION, PGPOOLVERSION);
+}
+
 static void usage(void)
 {
-	fprintf(stderr, "%s version %s(%s),\n",	PACKAGE, VERSION, PGPOOLVERSION);
+	fprintf(stderr, "%s version %s (%s),\n",	PACKAGE, VERSION, PGPOOLVERSION);
 	fprintf(stderr, "  a generic connection pool/replication/load balance server for PostgreSQL\n\n");
-	fprintf(stderr, "usage: pgpool [-c][-f config_file][-F pcp_config_file][-a hba_file][-n][-d]\n");
-	fprintf(stderr, "usage: pgpool [-f config_file][-F pcp_config_file][-a hba_file] [-m {s[mart]|f[ast]|i[mmediate]}] stop\n");
-	fprintf(stderr, "usage: pgpool [-f config_file][-F pcp_config_file][-a hba_file] reload\n");
-	fprintf(stderr, "usage: pgpool -h\n");
-	fprintf(stderr, "  config_file default path: %s/%s\n",DEFAULT_CONFIGDIR, POOL_CONF_FILE_NAME);
-	fprintf(stderr, "  pcp_config_file default path: %s/%s\n", DEFAULT_CONFIGDIR, PCP_PASSWD_FILE_NAME);
-	fprintf(stderr, "  hba_file default path:    %s/%s\n",DEFAULT_CONFIGDIR, HBA_CONF_FILE_NAME);
-	fprintf(stderr, "  -c: clears query cache. enable_query_cache must be on\n");
-	fprintf(stderr, "  -n: don't run in daemon mode. does not detatch control tty\n");
-	fprintf(stderr, "  -d: debug mode. lots of debug information will be printed\n");
-	fprintf(stderr, "  stop: stop pgpool\n");
-	fprintf(stderr, "  -h: print this help\n");
+	fprintf(stderr, "Usage:\n");
+	fprintf(stderr, "  pgpool [ -c] [ -f CONFIG_FILE ] [ -F PCP_CONFIG_FILE ] [ -a HBA_CONFIG_FILE ]\n");
+	fprintf(stderr, "         [ -n ] [ -d ]\n");
+	fprintf(stderr, "  pgpool [ -f CONFIG_FILE ] [ -F PCP_CONFIG_FILE ] [ -a HBA_CONFIG_FILE ]\n");
+	fprintf(stderr, "         [ -m SHUTDOWN-MODE ] stop\n");
+	fprintf(stderr, "  pgpool [ -f CONFIG_FILE ] [ -F PCP_CONFIG_FILE ] [ -a HBA_CONFIG_FILE ] reload\n\n");
+	fprintf(stderr, "Common options:\n");
+	fprintf(stderr, "  -a HBA_CONFIG_FILE  Sets the path to the pool_hba.conf configuration file\n");
+	fprintf(stderr, "                      (default: %s/%s)\n",DEFAULT_CONFIGDIR, HBA_CONF_FILE_NAME);
+	fprintf(stderr, "  -f CONFIG_FILE      Sets the path to the pgpool.conf configuration file\n");
+	fprintf(stderr, "                      (default: %s/%s)\n",DEFAULT_CONFIGDIR, POOL_CONF_FILE_NAME);
+	fprintf(stderr, "  -F PCP_CONFIG_FILE  Sets the path to the pcp.conf configuration file\n");
+	fprintf(stderr, "                      (default: %s/%s)\n",DEFAULT_CONFIGDIR, PCP_PASSWD_FILE_NAME);
+	fprintf(stderr, "  -h                  Prints this help\n\n");
+	fprintf(stderr, "Start options:\n");
+	fprintf(stderr, "  -c                  Clears query cache (enable_query_cache must be on)\n");
+	fprintf(stderr, "  -n                  Don't run in daemon mode, does not detach control tty\n");
+	fprintf(stderr, "  -d                  Debug mode\n\n");
+	fprintf(stderr, "Stop options:\n");
+	fprintf(stderr, "  -m SHUTDOWN-MODE    Can be \"smart\", \"fast\", or \"immediate\"\n\n");
+	fprintf(stderr, "Shutdown modes are:\n");
+	fprintf(stderr, "  smart       quit after all clients have disconnected\n");
+	fprintf(stderr, "  fast        quit directly, with proper shutdown\n");
+	fprintf(stderr, "  immediate   quit without complete shutdown; will lead to recovery on restart\n");
 }
 
 /*
-* detatch control ttys
+* detach control ttys
 */
 static void daemonize(void)
 {
 	int			i;
 	pid_t		pid;
+	int			fdlimit;
 
 	pid = fork();
 	if (pid == (pid_t) -1)
@@ -631,11 +667,16 @@ static void daemonize(void)
 
 	mypid = getpid();
 
+	chdir("/");
+
 	i = open("/dev/null", O_RDWR);
 	dup2(i, 0);
 	dup2(i, 1);
 	dup2(i, 2);
-	close(i);
+
+    fdlimit = sysconf(_SC_OPEN_MAX);
+    for (i = 3; i < fdlimit; i++)
+		close(i);
 
 	write_pid_file();
 }
@@ -679,11 +720,9 @@ static void stop_me(void)
 static int read_pid_file(void)
 {
 	FILE *fd;
-	char path[POOLMAXPATHLEN];
 	char pidbuf[128];
 
-	snprintf(path, sizeof(path), "%s/%s", pool_config->logdir, PID_FILE_NAME);
-	fd = fopen(path, "r");
+	fd = fopen(pool_config->pid_file_name, "r");
 	if (!fd)
 	{
 		return -1;
@@ -691,7 +730,7 @@ static int read_pid_file(void)
 	if (fread(pidbuf, 1, sizeof(pidbuf), fd) <= 0)
 	{
 		pool_error("could not read pid file as %s. reason: %s",
-				   path, strerror(errno));
+				   pool_config->pid_file_name, strerror(errno));
 		fclose(fd);
 		return -1;
 	}
@@ -705,15 +744,13 @@ static int read_pid_file(void)
 static void write_pid_file(void)
 {
 	FILE *fd;
-	char path[POOLMAXPATHLEN];
 	char pidbuf[128];
 
-	snprintf(path, sizeof(path), "%s/%s", pool_config->logdir, PID_FILE_NAME);
-	fd = fopen(path, "w");
+	fd = fopen(pool_config->pid_file_name, "w");
 	if (!fd)
 	{
 		pool_error("could not open pid file as %s. reason: %s",
-				   path, strerror(errno));
+				   pool_config->pid_file_name, strerror(errno));
 		pool_shmem_exit(1);
 		exit(1);
 	}
@@ -722,7 +759,7 @@ static void write_pid_file(void)
 	if (fclose(fd))
 	{
 		pool_error("could not write pid file as %s. reason: %s",
-				   path, strerror(errno));
+				   pool_config->pid_file_name, strerror(errno));
 		pool_shmem_exit(1);
 		exit(1);
 	}
@@ -768,8 +805,19 @@ pid_t fork_a_child(int unix_fd, int inet_fd, int id)
 
 	if (pid == 0)
 	{
-		close(pipe_fds[0]);
-		close(pipe_fds[1]);
+		/* Before we unconditionally closed pipe_fds[0] and pipe_fds[1]
+		 * here, which is apparently wrong since in the start up of
+		 * pgpool, pipe(2) is not called yet and it mistakenly closes
+		 * fd 0. Now we check the fd > 0 before close(), expecting
+		 * pipe returns fds greater than 0.  Note that we cannot
+		 * unconditionally remove close(2) calls since fork_a_child()
+		 * may be called *after* pgpool starting up.
+		 */
+		if (pipe_fds[0] > 0)
+		{
+			close(pipe_fds[0]);
+			close(pipe_fds[1]);
+		}
 
 		myargv = save_ps_display_args(myargc, myargv);
 
@@ -836,7 +884,13 @@ static int create_inet_domain_socket(const char *hostname, const int port)
 	status = bind(fd, (struct sockaddr *)&addr, len);
 	if (status == -1)
 	{
-		pool_error("bind() failed. reason: %s", strerror(errno));
+		char *host = "", *serv = "";
+		char hostname[NI_MAXHOST], servname[NI_MAXSERV];
+		if (getnameinfo((struct sockaddr *) &addr, len, hostname, sizeof(hostname), servname, sizeof(servname), 0) == 0) {
+			host = hostname;
+			serv = servname;
+		}
+		pool_error("bind(%s:%s) failed. reason: %s", host, serv, strerror(errno));
 		myexit(1);
 	}
 
@@ -872,7 +926,7 @@ static int create_unix_domain_socket(struct sockaddr_un un_addr_tmp)
 	status = bind(fd, (struct sockaddr *)&addr, len);
 	if (status == -1)
 	{
-		pool_error("bind() failed. reason: %s", strerror(errno));
+		pool_error("bind(%s) failed. reason: %s", addr.sun_path, strerror(errno));
 		myexit(1);
 	}
 
@@ -891,9 +945,14 @@ static int create_unix_domain_socket(struct sockaddr_un un_addr_tmp)
 	return fd;
 }
 
+static void myunlink(const char* path)
+{
+	if (unlink(path) == 0) return;
+	pool_error("unlink(%s) failed: %s", path, strerror(errno));
+}
+
 static void myexit(int code)
 {
-	char path[POOLMAXPATHLEN];
 	int i;
 	
 	if (getpid() != mypid)
@@ -917,10 +976,9 @@ static void myexit(int code)
 		POOL_SETMASK(&UnBlockSig);
 	}
 	
-	unlink(un_addr.sun_path);
-	unlink(pcp_un_addr.sun_path);
-	snprintf(path, sizeof(path), "%s/%s", pool_config->logdir, PID_FILE_NAME);
-	unlink(path);
+	myunlink(un_addr.sun_path);
+	myunlink(pcp_un_addr.sun_path);
+	myunlink(pool_config->pid_file_name);
 
 	pool_shmem_exit(code);
 	exit(code);
@@ -1036,6 +1094,30 @@ static RETSIGTYPE exit_handler(int sig)
 	myexit(0);
 }
 
+/*
+ * calculate next master node id
+ */
+static int get_next_master_node(void)
+{
+	int i;
+	for (i=0;i<pool_config->backend_desc->num_backends;i++)
+	{
+		/*
+		 * Do not use VALID_BACKEND macro in raw mode.
+		 * VALID_BACKEND return true only if the argument is master
+		 * node id. In other words, standby nodes are false. So need
+		 * to check backend status without VALID_BACKEND.
+		 */
+		if (RAW_MODE)
+		{
+			if (BACKEND_INFO(i).backend_status == CON_CONNECT_WAIT)
+				break;
+		}
+		else if (VALID_BACKEND(i))
+			break;
+	}
+	return i;
+}
 
 /*
  * handle SIGUSR1
@@ -1055,11 +1137,15 @@ static RETSIGTYPE failover_handler(int sig)
  */
 static void failover(void)
 {
-	int node_id;
 	int i;
+	int node_id;
+	int new_master;
+	int nodes[MAX_NUM_BACKENDS];
 
 	pool_debug("failover_handler called");
 
+	memset(nodes, 0, sizeof(int) * MAX_NUM_BACKENDS);
+
 	/*
 	 * this could happen in a child process if a signal has been sent
 	 * before resetting signal handler
@@ -1105,7 +1191,7 @@ static void failover(void)
 	 * if not in replication mode/master slave mode, we treat this a restart request.
 	 * otherwise we need to check if we have already failovered.
 	 */
-	pool_debug("failover_handler: starting to selec new master node");
+	pool_debug("failover_handler: starting to select new master node");
 	switching = 1;
 	node_id = Req_info->node_id[0];
 
@@ -1120,6 +1206,7 @@ static void failover(void)
 			pool_error("failover_handler: invalid node_id %d status:%d MAX_NUM_BACKENDS: %d", node_id,
 					   BACKEND_INFO(node_id).backend_status, MAX_NUM_BACKENDS);
 			kill(pcp_pid, SIGUSR2);
+			switching = 0;
 			return;
 		}
 
@@ -1144,7 +1231,8 @@ static void failover(void)
 
 
 				BACKEND_INFO(Req_info->node_id[i]).backend_status = CON_DOWN;	/* set down status */
-				trigger_failover_command(Req_info->node_id[i], pool_config->failover_command);
+				/* save down node */
+				nodes[Req_info->node_id[i]] = 1;
 				cnt++;
 			}
 		}
@@ -1154,25 +1242,37 @@ static void failover(void)
 			pool_log("failover: no backends are degenerated");
 			pool_semaphore_unlock(REQUEST_INFO_SEM);
 			kill(pcp_pid, SIGUSR2);
+			switching = 0;
 			return;
 		}
 	}
 
-	for (i=0;i<pool_config->backend_desc->num_backends;i++)
-	{
-		if (VALID_BACKEND(i))
-			break;
-	}
+	new_master = get_next_master_node();
 
-	if (i == pool_config->backend_desc->num_backends)
+	if (new_master == pool_config->backend_desc->num_backends)
 	{
 		pool_error("failover_handler: no valid DB node found");
 	}
+
+/*
+ * Before we tried to minimize restarting pgpool to protect existing
+ * connections from clients to pgpool children. What we did here was,
+ * if children other than master went down, we did not fail over.
+ * This is wrong. Think about following scenario. If someone
+ * accidentally plugs out the network cable, the TCP/IP stack keeps
+ * retrying for long time (typically 2 hours). The only way to stop
+ * the retry is restarting the process.  Bottom line is, we need to
+ * restart all children in any case.  See pgpool-general list posting
+ * "TCP connections are *not* closed when a backend timeout" on Jul 13
+ * 2008 for more details.
+ */
+
+#ifdef NOT_USED
 	else
 	{
-		if (Req_info->master_node_id == i && *InRecovery == 0)
+		if (Req_info->master_node_id == new_master && *InRecovery == 0)
 		{
-			pool_log("failover_handler: do not restart pgpool. same master node %d was selected", i);
+			pool_log("failover_handler: do not restart pgpool. same master node %d was selected", new_master);
 			if (Req_info->kind == NODE_UP_REQUEST)
 			{
 				pool_log("failback done. reconnect host %s(%d)",
@@ -1186,16 +1286,21 @@ static void failover(void)
 						 BACKEND_INFO(node_id).backend_port);
 			}
 
+			/* exec failover_command */
+			for (i = 0; i < pool_config->backend_desc->num_backends; i++)
+			{
+				if (nodes[i])
+					trigger_failover_command(i, pool_config->failover_command);
+			}
+
 			pool_semaphore_unlock(REQUEST_INFO_SEM);
 			switching = 0;
 			kill(pcp_pid, SIGUSR2);
+			switching = 0;
 			return;
 		}
-
-		pool_log("failover_handler: set new master node: %d", i);
-		Req_info->master_node_id = i;
 	}
-
+#endif
 	/* kill all children */
 	for (i = 0; i < pool_config->num_init_children; i++)
 	{
@@ -1207,6 +1312,16 @@ static void failover(void)
 		}
 	}
 
+	/* exec failover_command */
+	for (i = 0; i < pool_config->backend_desc->num_backends; i++)
+	{
+		if (nodes[i])
+			trigger_failover_command(i, pool_config->failover_command);
+	}
+
+	pool_log("failover_handler: set new master node: %d", new_master);
+	Req_info->master_node_id = new_master;
+
 /* no need to wait since it will be done in reap_handler */
 #ifdef NOT_USED
 	while (wait(NULL) > 0)
@@ -1240,6 +1355,10 @@ static void failover(void)
 	}
 
 	switching = 0;
+
+	/* kick wakeup_handler in pcp_child to notice that
+	 * faiover/failback done
+	 */
 	kill(pcp_pid, SIGUSR2);
 }
 
@@ -1285,7 +1404,7 @@ int health_check(void)
 
 	for (i=0;i<pool_config->backend_desc->num_backends;i++)
 	{
-		pool_debug("health_check: %d the DB node status: %d", i, BACKEND_INFO(i).backend_status);
+		pool_debug("health_check: %d th DB node status: %d", i, BACKEND_INFO(i).backend_status);
 
 		if (BACKEND_INFO(i).backend_status == CON_UNUSED ||
 			BACKEND_INFO(i).backend_status == CON_DOWN)
@@ -1318,7 +1437,11 @@ int health_check(void)
 
 		read(fd, &kind, 1);
 
-		if (write(fd, "X", 1) < 0)
+		/*
+		 * If a backend raised a FATAL error(max connections error or
+		 * starting up error?), do not send a Terminate message.
+		 */
+		if ((kind != 'E') && (write(fd, "X", 1) < 0))
 		{
 			pool_error("health check failed during write. host %s at port %d is down. reason: %s. Perhaps wrong health check user?",
 					   BACKEND_INFO(i).backend_hostname,
@@ -1426,7 +1549,6 @@ static void reaper(void)
 	int i;
 
 	pool_debug("reap_handler called");
-	sigchld_request = 0;
 
 	if (exiting)
 	{
@@ -1436,10 +1558,13 @@ static void reaper(void)
 
 	if (switching)
 	{
-		pool_debug("reap_handler: exited due to swicting");
+		pool_debug("reap_handler: exited due to switching");
 		return;
 	}
 
+	/* clear SIGCHLD request */
+	sigchld_request = 0;
+
 #ifdef HAVE_WAITPID
 	pool_debug("reap_handler: call waitpid");
 	while ((pid = waitpid(-1, &status, WNOHANG)) > 0)
@@ -1599,10 +1724,20 @@ static void kill_all_children(int sig)
 			kill(pid, sig);
 		}
 	}
+
+	/* make PCP process reload as well */
+	if (sig == SIGHUP)
+		kill(pcp_pid, sig);
 }
 
 /*
- * pool_pause: A process pauses by select().
+ * pause in a period specified by timeout. If any data is coming
+ * through pipe_fds[0], that means one of: failover request(SIGUSR1),
+ * SIGCHLD received, children wake up request(SIGUSR2 used in on line
+ * recovery processing) or config file reload request(SIGHUP) has been
+ * occurred.  In this case this function returns 1.
+ * otherwise 0: (no signal event occurred), -1: (error)
+ * XXX: is it ok that select(2) error is ignored here?
  */
 static int pool_pause(struct timeval *timeout)
 {
@@ -1619,8 +1754,12 @@ static int pool_pause(struct timeval *timeout)
 }
 
 /*
- * pool_pause: A process sleep using pool_pause().
- *             If a signal event occurs, it raises signal handler.
+ * sleep for seconds specified by "second".  Unlike pool_pause(), this
+ * function guarantees that it will sleep for specified seconds.  This
+ * function uses pool_pause() internally. If it informs that there is
+ * a pending signal event, they are processed using CHECK_REQUEST
+ * macro. Note that most of these processes are done while all signals
+ * are blocked.
  */
 static void pool_sleep(unsigned int second)
 {
@@ -1631,8 +1770,7 @@ static void pool_sleep(unsigned int second)
 	sleep_time.tv_usec = current_time.tv_usec;
 
 	POOL_SETMASK(&UnBlockSig);
-	while (sleep_time.tv_sec > current_time.tv_sec ||
-		   sleep_time.tv_usec > current_time.tv_usec)
+	while (sleep_time.tv_sec > current_time.tv_sec)
 	{
 		struct timeval timeout;
 		int r;
@@ -1695,7 +1833,7 @@ static int trigger_failover_command(int node, const char *command_line)
 		return -1;
 
 	buf[1] = '\0';
-	pool_memory = pool_memory_create();
+	pool_memory = pool_memory_create(PREPARE_BLOCK_SIZE);
 	if (!pool_memory)
 	{
 		pool_error("trigger_failover_command: pool_memory_create() failed");
@@ -1730,6 +1868,16 @@ static int trigger_failover_command(int node, const char *command_line)
 						string_append_char(exec_cmd, info->backend_hostname);
 						break;
 
+					case 'm': /* new master node id */
+						snprintf(port_buf, sizeof(port_buf), "%d", get_next_master_node());
+						string_append_char(exec_cmd, port_buf);
+						break;
+
+					case 'M': /* old master node id */
+						snprintf(port_buf, sizeof(port_buf), "%d", MASTER_NODE_ID);
+						string_append_char(exec_cmd, port_buf);
+						break;
+
 					case '%': /* escape */
 						string_append_char(exec_cmd, "%");
 						break;
