@@ -1,5 +1,21 @@
 /*
- * $Header: /cvsroot/pgpool/pgpool-II/pcp/pcp_node_count.c,v 1.1.1.1 2006/09/08 03:36:49 t-ishii Exp $
+ * $Header: /cvsroot/pgpool/pgpool-II/pcp/pcp_node_count.c,v 1.3 2008/12/31 10:25:40 t-ishii Exp $
+ *
+ * pgpool: a language independent connection pool server for PostgreSQL 
+ * written by Tatsuo Ishii
+ *
+ * Copyright (c) 2003-2008	PgPool Global Development Group
+ *
+ * Permission to use, copy, modify, and distribute this software and
+ * its documentation for any purpose and without fee is hereby
+ * granted, provided that the above copyright notice appear in all
+ * copies and that both that copyright notice and this permission
+ * notice appear in supporting documentation, and that the name of the
+ * author not be used in advertising or publicity pertaining to
+ * distribution of the software without specific, written prior
+ * permission. The author makes no representations about the
+ * suitability of this software for any purpose.  It is provided "as
+ * is" without express or implied warranty.
  *
  * Client program to send "node count" command.
  */
@@ -7,6 +23,7 @@
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
+#include <unistd.h>
 
 #include "pcp.h"
 
@@ -22,53 +39,64 @@ main(int argc, char **argv)
 	char user[MAX_USER_PASSWD_LEN];
 	char pass[MAX_USER_PASSWD_LEN];
 	int node_count;
+	int ch;
 
-	if (argc == 2 && (strcmp(argv[1], "-h") == 0) )
-	{
-		usage();
-		exit(0);
+	while ((ch = getopt(argc, argv, "hd")) != -1) {
+		switch (ch) {
+		case 'd':
+			pcp_enable_debug();
+			break;
+
+		case 'h':
+		case '?':
+		default:
+			usage();
+			exit(0);
+		}
 	}
+	argc -= optind;
+	argv += optind;
 
-	if (argc != 6) {
+	if (argc != 5) {
 		errorcode = INVALERR;
 		pcp_errorstr(errorcode);
 		myexit(errorcode);
 	}
 
-	timeout = atol(argv[1]);
+	timeout = atol(argv[0]);
 	if (timeout < 0) {
 		errorcode = INVALERR;
 		pcp_errorstr(errorcode);
 		myexit(errorcode);
 	}
 
-	if (strlen(argv[2]) >= MAX_DB_HOST_NAMELEN) {
+	if (strlen(argv[1]) >= MAX_DB_HOST_NAMELEN) {
 		errorcode = INVALERR;
 		pcp_errorstr(errorcode);
 		myexit(errorcode);
 	}
-	strcpy(host, argv[2]);
+	strcpy(host, argv[1]);
 
-	port = atoi(argv[3]);
+	port = atoi(argv[2]);
 	if (port <= 1024 || port > 65535) {
 		errorcode = INVALERR;
 		pcp_errorstr(errorcode);
 		myexit(errorcode);
 	}
 
+	if (strlen(argv[3]) >= MAX_USER_PASSWD_LEN) {
+		errorcode = INVALERR;
+		pcp_errorstr(errorcode);
+		myexit(errorcode);
+	}
+	strcpy(user, argv[3]);
+
 	if (strlen(argv[4]) >= MAX_USER_PASSWD_LEN) {
 		errorcode = INVALERR;
 		pcp_errorstr(errorcode);
 		myexit(errorcode);
 	}
-	strcpy(user, argv[4]);
-
-	if (strlen(argv[5]) >= MAX_USER_PASSWD_LEN) {
-		errorcode = INVALERR;
-		pcp_errorstr(errorcode);
-		myexit(errorcode);
-	}
-	strcpy(pass, argv[5]);
+	strcpy(pass, argv[4]);
 
 	pcp_set_timeout(timeout);
 
@@ -96,8 +124,9 @@ static void
 usage(void)
 {
 	fprintf(stderr, "pcp_node_count - display the total number of nodes under pgpool-II's control\n\n");
-	fprintf(stderr, "Usage: pcp_node_count timeout hostname port# username password\n");
+	fprintf(stderr, "Usage: pcp_node_count [-d] timeout hostname port# username password\n");
 	fprintf(stderr, "Usage: pcp_node_count -h\n\n");
+	fprintf(stderr, "  -d       - enable debug message (optional)\n");
 	fprintf(stderr, "  timeout  - connection timeout value in seconds. command exits on timeout\n");
 	fprintf(stderr, "  hostname - pgpool-II hostname\n");
 	fprintf(stderr, "  port#    - pgpool-II port number\n");
