@@ -1,11 +1,11 @@
 /* -*-pgsql-c-*- */
 /*
- * $Header: /cvsroot/pgpool/pgpool-II/pool_auth.c,v 1.7 2007/11/16 02:52:51 y-asaba Exp $
+ * $Header: /cvsroot/pgpool/pgpool-II/pool_auth.c,v 1.15 2009/02/24 02:20:40 t-ishii Exp $
  *
  * pgpool: a language independent connection pool server for PostgreSQL 
  * written by Tatsuo Ishii
  *
- * Copyright (c) 2003-2007	PgPool Global Development Group
+ * Copyright (c) 2003-2008	PgPool Global Development Group
  *
  * Permission to use, copy, modify, and distribute this software and
  * its documentation for any purpose and without fee is hereby
@@ -84,9 +84,14 @@ int pool_do_auth(POOL_CONNECTION *frontend, POOL_CONNECTION_POOL *cp)
 	}
 
 	/*
-	 * message length (v3 only) */
+	 * message length (v3 only)
+	 */
 	if (protoMajor == PROTO_MAJOR_V3 && pool_read_message_length(cp) < 0)
 	{
+		pool_error("Failed to read the authentication packet length. \
+This is likely caused by the inconsistency of auth method among DB nodes. \
+In this case you can check the previous error messages (hint: length field) \
+from pool_read_message_length and recheck the pg_hba.conf settings.");
 		return -1;
 	}
 
@@ -179,7 +184,7 @@ int pool_do_auth(POOL_CONNECTION *frontend, POOL_CONNECTION_POOL *cp)
 	/* md5 authentication? */
 	else if (authkind == 5)
 	{
-		if (NUM_BACKENDS > 1)
+		if (!RAW_MODE && NUM_BACKENDS > 1)
 		{
 			pool_send_error_message(frontend, protoMajor, AUTHFAIL_ERRORCODE,
 									"MD5 authentication is unsupported in replication, master-slave and parallel modes.",
@@ -322,7 +327,8 @@ int pool_do_auth(POOL_CONNECTION *frontend, POOL_CONNECTION_POOL *cp)
 				pool_error("pool_do_auth: failed to read pid in slot %d", i);
 				return -1;
 			}
-			CONNECTION_SLOT(cp, i)->pid = pid;
+
+			CONNECTION_SLOT(cp, i)->pid = cp->info[i].pid = pid;
 
 			/* read key */
 			if (pool_read(CONNECTION(cp, i), &key, sizeof(key)) < 0)
@@ -330,7 +336,8 @@ int pool_do_auth(POOL_CONNECTION *frontend, POOL_CONNECTION_POOL *cp)
 				pool_error("pool_do_auth: failed to read key in slot %d", i);
 				return -1;
 			}
-			CONNECTION_SLOT(cp, i)->key = key;
+			CONNECTION_SLOT(cp, i)->key = cp->info[i].key = key;
+	
 		}
 	}
 
@@ -947,11 +954,19 @@ int pool_read_message_length(POOL_CONNECTION_POOL *cp)
 	int length, length0;
 	int i;
 
-	length0 = 0;
+	/* read message from master node */
+	status = pool_read(CONNECTION(cp, MASTER_NODE_ID), &length0, sizeof(length0));
+	if (status < 0)
+	{
+		pool_error("pool_read_message_length: error while reading message length in slot %d", MASTER_NODE_ID);
+		return -1;
+	}
+	length0 = ntohl(length0);
+	pool_debug("pool_read_message_length: slot: %d length: %d", MASTER_NODE_ID, length0);
 
 	for (i=0;i<NUM_BACKENDS;i++)
 	{
-		if (!VALID_BACKEND(i))
+		if (!VALID_BACKEND(i) || IS_MASTER_NODE_ID(i))
 		{
 			continue;
 		}
@@ -966,27 +981,20 @@ int pool_read_message_length(POOL_CONNECTION_POOL *cp)
 		length = ntohl(length);
 		pool_debug("pool_read_message_length: slot: %d length: %d", i, length);
 
-		if (IS_MASTER_NODE_ID(i))
+		if (length != length0)
 		{
-			length0 = length;
-		}
-		else
-		{
-			if (length != length0)
-			{
-				pool_error("pool_read_message_length: message length (%d) in slot %d does not match with slot 0", length, i);
-				return -1;
-			}
+			pool_error("pool_read_message_length: message length (%d) in slot %d does not match with slot 0(%d)", length, i, length0);
+			return -1;
 		}
 	}
 
-	if (length < 0)
+	if (length0 < 0)
 	{
 		pool_error("pool_read_message_length: invalid message length (%d)", length);
 		return -1;
 	}
 
-	return length;
+	return length0;
 }
 
 /*
@@ -1001,11 +1009,20 @@ int *pool_read_message_length2(POOL_CONNECTION_POOL *cp)
 	int i;
 	static int length_array[MAX_CONNECTION_SLOTS];
 
-	length0 = 0;
+	/* read message from master node */
+	status = pool_read(CONNECTION(cp, MASTER_NODE_ID), &length0, sizeof(length0));
+	if (status < 0)
+	{
+		pool_error("pool_read_message_length2: error while reading message length in slot %d", MASTER_NODE_ID);
+		return NULL;
+	}
 
+	length0 = ntohl(length0);
+	length_array[MASTER_NODE_ID] = length0;
+	pool_debug("pool_read_message_length2: master slot: %d length: %d", MASTER_NODE_ID, length0);
 	for (i=0;i<NUM_BACKENDS;i++)
 	{
-		if (VALID_BACKEND(i))
+		if (VALID_BACKEND(i) && !IS_MASTER_NODE_ID(i))
 		{
 			status = pool_read(CONNECTION(cp, i), &length, sizeof(length));
 			if (status < 0)
@@ -1017,16 +1034,9 @@ int *pool_read_message_length2(POOL_CONNECTION_POOL *cp)
 			length = ntohl(length);
 			pool_debug("pool_read_message_length2: master slot: %d length: %d", i, length);
 
-			if (IS_MASTER_NODE_ID(i))
+			if (length != length0)
 			{
-				length0 = length;
-			}
-			else
-			{
-				if (length != length0)
-				{
-					pool_error("pool_read_message_length2: message length (%d) in slot %d does not match with slot 0", length, i);
-				}
+				pool_log("pool_read_message_length2: message length (%d) in slot %d does not match with slot 0(%d)", length, i, length0);
 			}
 
 			if (length < 0)
